/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.tiff.provider;

import it.tidalwave.metadata.text.APEXApertureFormat;
import it.tidalwave.metadata.text.APEXShutterFormat;
import it.tidalwave.metadata.text.ApertureFormat;
import it.tidalwave.metadata.text.EVFormat;
import it.tidalwave.metadata.text.FocalFormat;
import it.tidalwave.metadata.text.ISOFormat;
import it.tidalwave.metadata.text.ShutterFormat;
import it.tidalwave.metadata.text.CFAPatternFormat;
import it.tidalwave.metadata.text.DeweyFormat;
import it.tidalwave.metadata.text.EXIFFlashFormat;
import javax.annotation.Nonnull;
    
/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class TIFFFormatProviderSupport<Item> extends DirectoryFormatProviderSupport<Item>
  {
    public TIFFFormatProviderSupport (@Nonnull final Class<Item> itemClass,
                                      @Nonnull final String displayName)
      {
        super(itemClass, displayName);
        
        // TODO: move to a configuration file?
        formatMap.put("apertureValue",       new APEXApertureFormat());
        formatMap.put("maxApertureValue",    new APEXApertureFormat());
        formatMap.put("exposureBiasValue",   new EVFormat(EVFormat.Fractions.UNICODE));
        formatMap.put("exposureTime",        new ShutterFormat());
        formatMap.put("FNumber",             new ApertureFormat(ApertureFormat.F.UNICODE));
        formatMap.put("ISOSpeedRatings",     new ISOFormat());
        formatMap.put("focalLength",         new FocalFormat());
        formatMap.put("focalLength35mmFilm", new FocalFormat());
        formatMap.put("shutterSpeedValue",   new APEXShutterFormat());
        formatMap.put("EXIFCFAPattern",      new CFAPatternFormat());
        formatMap.put("EXIFVersion",         new DeweyFormat());
        formatMap.put("DNGBackwardVersion",  new DeweyFormat());
        formatMap.put("DNGVersion",          new DeweyFormat());
        formatMap.put("TIFF_EPStandardID",   new DeweyFormat());
        formatMap.put("flashPixVersion",     new DeweyFormat());
        formatMap.put("flash",               new EXIFFlashFormat());
      }
  }
